/////
// BVView.cpp
// implement file for BVView class
//
// Hideki Naito
// last modified : 1999/08/07
//

#include	<stdio.h>
#include	<unistd.h>
#include	"BVView.h"
#include	"apmdrv.h"


/////
// constants
//

// messages
#define	MSG_CHECKSTS	'ckst'

// transparent color
#define	TRAN			B_TRANSPARENT_8_BIT

// Pulse calling rate
#define	PULSE_RATE		2000000

// AC line status
enum
{
	ACL_OFFLINE= 0,		// off line (use battery)
	ACL_ONLINE,			// on line (use AC adapter)
	ACL_BACKPWR,		// user backup-power
	ACL_UNKNOWN			// unknown
};


/////
// icon bitmap
//
unsigned char pat[][16*16] =
{
	{// off line
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x18, 0x18, 0x18, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x18, 0x10, 0x00, 0x18,
		0x18, 0x18, 0x00, 0x10, 0x18, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN,
		0x00, 0x00, 0x18, 0x10, 0x10, 0x10, 0x10, 0x10, 0x18, 0x00, 0x00,
		TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x25, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x25, 0x25, 0x00, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
		0x20, 0x25, 0x25, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x25, 0x25, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20,
		0x25, 0x25, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x20, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x20, 0x25, 0x25, 0x00, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x20, 0x25,
		0x25, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x20, 0x20, 0x20,
		0x20, 0x20, 0x20, 0x20, 0x25, 0x25, 0x00, TRAN, TRAN, TRAN, TRAN,
		TRAN, 0x00, 0x25, 0x20, 0x20, 0x20, 0x20, 0x20, 0x25, 0x25, 0x25,
		0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x25, 0x25, 0x25,
		0x25, 0x25, 0x25, 0x25, 0x00, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, TRAN,
		TRAN, TRAN, TRAN
	}
	,
	{// on line
		TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, TRAN, 0x00, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x18, 0x10,
		0x00, 0x18, 0x10, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0x18, 0x10, 0x00, 0x18, 0x10, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x18, 0x10, 0x5e,
		0x18, 0x10, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		0x00, 0x5e, 0x10, 0x8f, 0x00, 0x10, 0x10, 0x7f, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x52, 0x52, 0x52, 0x52, 0x52,
		0x5e, 0x3e, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x52, 0x52, 0x52, 0x52, 0x52, 0x5e, 0x3e, 0x00, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0x52, 0x52, 0x52, 0x52, 0x52, 0x5e,
		0x3e, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x52,
		0x52, 0x52, 0x52, 0x5e, 0x3e, 0x5e, 0x00, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0x5e, 0x3e, 0x5e, 0x3e, 0x5e, 0x00,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x00, 0x00, 0x00, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x52, 0x00, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x52, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0x52, 0x52, 0x00, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x00, 0x52, 0x52, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x52, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN
	}
	,
	{// backup power
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x18, 0x18, 0x18, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x18, 0x10, 0x00, 0x18,
		0x18, 0x18, 0x00, 0x10, 0x18, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN,
		0x00, 0x00, 0x18, 0x10, 0x10, 0x10, 0x10, 0x10, 0x18, 0x00, 0x00,
		TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x2a, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x2f, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00,
		0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x2f, 0x2f, 0x00, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa, 0xfa, 0xfa, 0x2a, 0x2a, 0xfa,
		0x2a, 0x2f, 0xfa, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa,
		0x2a, 0x2a, 0xfa, 0x2a, 0xfa, 0xfa, 0xfa, 0x2f, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, 0x00, 0xfa, 0xfa, 0xfa, 0xfa, 0x2a, 0xfa, 0xfa,
		0xfa, 0x2f, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa, 0x2a,
		0x2a, 0xfa, 0x2a, 0xfa, 0x2a, 0xfa, 0xfa, 0x00, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0xfa, 0xfa, 0xfa, 0xfa, 0x2a, 0xfa, 0x2a, 0x2f,
		0xfa, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x2a, 0x2a, 0x2a,
		0x2a, 0x2a, 0x2a, 0x2a, 0x2f, 0x2f, 0x00, TRAN, TRAN, TRAN, TRAN,
		TRAN, 0x00, 0x2f, 0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x2f, 0x2f, 0x2f,
		0x00, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x2f, 0x2f, 0x2f,
		0x2f, 0x2f, 0x2f, 0x2f, 0x00, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, TRAN,
		TRAN, TRAN, TRAN
	}
	,
	{// unknown
		TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x00, 0x00, 0x00, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa,
		0xfa, 0xfa, 0xfa, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0xfa, 0xfa, 0x8b, 0x00, 0xfa, 0xfa, 0x8b, 0x00,
		0x10, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa, 0xfa, 0x8b, 0x00,
		0x10, 0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN, TRAN,
		0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10, 0x00, 0xfa, 0xfa, 0x8b, 0x00,
		0x10, TRAN, TRAN, TRAN, TRAN, 0x00, 0x8b, 0x8b, 0x8b, 0x00, 0x10,
		0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN, TRAN, TRAN,
		0x00, 0x00, 0x00, TRAN, TRAN, 0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa,
		0xfa, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, 0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa, 0xfa, 0x8b, 0x00,
		0x10, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x00, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		0x00, 0x00, 0x00, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0xfa, 0xfa, 0x8b, 0x00, 0x10, TRAN,
		TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, 0x00, 0x8b,
		0x8b, 0x8b, 0x00, 0x10, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN, TRAN, 0x00, 0x00, 0x00, TRAN, TRAN, TRAN, TRAN,
		TRAN, TRAN, TRAN
	}
};


/////
// signatures
//
extern const char *app_sig;


/////
// constructor for BVView class
//
BVView::BVView(
	BRect	frame)
	: BView(frame, "bvview", (B_FOLLOW_LEFT | B_FOLLOW_TOP), B_WILL_DRAW)
{
	this->mr = NULL;
	this->icon = NULL;
}


/////
// constructor for MCMiniIcon(for Replicant)
//
BVView::BVView(
	BMessage	*msg)
	: BView(msg)
{
	this->SetViewColor(189, 189, 189);

	// set default status
	this->ac_line = ACL_UNKNOWN;
	this->bt_per = 0;

	// set default icon
	this->icon = new BBitmap(BRect(0, 0, 15, 15), B_COLOR_8_BIT);
	this->icon->SetBits(pat[ACL_UNKNOWN], 256, 0, B_COLOR_8_BIT);

	// font setting
	BFont		font;
	font_height	fh;
	this->GetFont(&font);
	font.GetHeight(&fh);
	this->baseline = fh.ascent;
	this->SetFont(&font);
}


/////
// destructor for BVView
//
BVView::~BVView()
{
	delete this->mr;
	delete this->icon;
}


/////
// instanciate this view(for Replicant)
//
BVView *BVView::Instantiate(
	BMessage	*data)
{
	if(validate_instantiation(data, "BVView") == false)
	{
		return NULL;
	}
	return new BVView(data);
}


/////
// archiving object(for Replicant)
//
status_t BVView::Archive(
	BMessage	*data,
	bool		deep) const
{
	BView::Archive(data, deep);
	data->AddString("add_on", app_sig);
	return B_NO_ERROR;
}


/////
// view attached to window
//
void BVView::AttachedToWindow()
{
	this->SetViewColor(this->Parent()->ViewColor());
	this->SetDrawingMode(B_OP_ALPHA);
	this->mr = new BMessageRunner(BMessenger(this),
		new BMessage(MSG_CHECKSTS), PULSE_RATE);
}


/////
// message hook
//
void BVView::MessageReceived(
	BMessage	*msg)
{
	if(msg->what == MSG_CHECKSTS)
	{
		this->CheckStatus();
		this->Invalidate();
	}
	else
	{
		BView::MessageReceived(msg);
	}
}


/////
// redraw hook
//
void BVView::Draw(
	BRect	rect)
{
	char	out[5];

	this->DrawBitmap(this->icon);
	if(this->bt_per <= 100)
	{
		sprintf(out, "%3d%%", this->bt_per);
	}
	else
	{
		sprintf(out, "---%%");
	}
	this->DrawString(out, BPoint(15, this->baseline));
	return;
}


/////
// check battery status
//
void BVView::CheckStatus()
{
	int				fd, result;
	uint16			regs[6] = {0, 0, 0, 0, 0, 0};
	static uint8	pref = ACL_UNKNOWN;

	// open apm driver
	fd = open (APM_DEVICE_PATH, O_RDONLY);
	if(fd >= 0)
	{
		// get status
		regs[0] = APM_FUNC_OFFSET + APM_FUNC_GET_POWER_STATUS;
		regs[1] = APM_DEVICE_ALL;
		result = ioctl (fd, APM_BIOS_CALL, regs);
		this->ac_line = regs[1] >> 8;
		if(this->ac_line > ACL_UNKNOWN) this->ac_line = ACL_UNKNOWN;
		this->bt_per = regs[2] & 255;
		close(fd);
	}

	// change ac_line icon
	if(this->ac_line != pref)
	{
		this->icon->SetBits(pat[this->ac_line], 256, 0, B_COLOR_8_BIT);
		pref = this->ac_line;
	}
}
